@@ -245,7 +245,8 @@ int LocalWorker::stopDetachedWorker()
 
 int LocalWorker::stop()
 {
-    stopDetachedWorker();
+    if (getConfig().isDetached() && m_pDetached)
+        return -1;
     pid_t pid;
     int ret;
     PidList::iterator iter;
@@ -323,7 +324,6 @@ int LocalWorker::tryRestart()
 
 int LocalWorker::restart()
 {
-    stopDetachedWorker();
     setLastRestart(DateTime::s_curTime);
     clearCurConnPool();
     if ((getConfigPointer()->getURL()) &&
