@@ -113,7 +113,21 @@ void LocalWorkerConfig::setRLimits(const RLimits *pRLimits)
 }
 
 
-int LocalWorkerConfig::checkExtAppSelfManagedAndFixEnv()
+#define DETACH_MODE_MIN_MAX_IDLE 30
+#define DETACH_MODE_DEFAULT_MAX_IDLE 60
+static void setDetachedAppEnv(Env *pEnv, int max_idle)
+{
+    char achBuf[200];
+    snprintf(achBuf, 200, "LSAPI_PPID_NO_CHECK=1");
+    pEnv->add(achBuf);
+    snprintf(achBuf, 200, "LSAPI_PGRP_MAX_IDLE=%d", max_idle);
+    pEnv->add(achBuf);
+    snprintf(achBuf, 200,  "LSAPI_KEEP_LISTEN=2");
+    pEnv->add(achBuf);
+}
+
+
+int LocalWorkerConfig::checkExtAppSelfManagedAndFixEnv(int maxIdleTime)
 {
     static const char *instanceEnv[] =
     {
@@ -152,6 +166,14 @@ int LocalWorkerConfig::checkExtAppSelfManagedAndFixEnv()
         selfManaged = 1;
     }
 
+    
+    if (isDetached())
+    {
+        setDetachedAppEnv(pEnv,
+                          (maxIdleTime > DETACH_MODE_MIN_MAX_IDLE)
+                          ? maxIdleTime : DETACH_MODE_MIN_MAX_IDLE);
+    }
+    
     pEnv->add(0, 0, 0, 0);
     return selfManaged;
 }
@@ -189,8 +211,8 @@ int LocalWorkerConfig::config(const XmlNode *pNode)
     setUmask(umakeVal);
 
     setRunOnStartUp(ConfigCtx::getCurConfigCtx()->getLongValue(pNode,
-                    "runOnStartUp", 0, 2, 0));
-
+                    "runOnStartUp", 0, 3, 3));
+        
     RLimits limits;
     if (ExtAppRegistry::getRLimits() != NULL)
         limits = *(ExtAppRegistry::getRLimits());
@@ -216,7 +238,11 @@ int LocalWorkerConfig::config(const XmlNode *pNode)
                 instances);
     }
     setInstances(instances);
-    selfManaged = checkExtAppSelfManagedAndFixEnv();
+    
+    
+    long maxIdle = ConfigCtx::getCurConfigCtx()->getLongValue(pNode,
+                   "extMaxIdleTime", -1, INT_MAX, DETACH_MODE_DEFAULT_MAX_IDLE);
+    selfManaged = checkExtAppSelfManagedAndFixEnv(maxIdle);
     setSelfManaged(selfManaged);
     if ((instances != 1) &&
         (getMaxConns() > instances))
