@@ -527,15 +527,8 @@ int HttpCgiTool::buildCommonEnv(IEnv *pEnv, HttpSession *pSession)
                   pStr->c_str(), pStr->len() - 1);
         
         
-    //We nee dto remove the '[' and ']' from the IPv6
-    const char *sIp = pSession->getPeerAddrString();
-    int len = pSession->getPeerAddrStrLen();
-    if (len > 2 && *sIp == '[' && sIp[len - 1] == ']')
-    {
-        ++sIp;
-        len -= 2;
-    }   
-    pEnv->add("REMOTE_ADDR", 11, sIp, len);
+    pEnv->add("REMOTE_ADDR", 11, pSession->getPeerAddrString(),
+              pSession->getPeerAddrStrLen());
 
     n = ls_snprintf(buf, 10, "%hu", pSession->getRemotePort());
     pEnv->add("REMOTE_PORT", 11, buf, n);
