@@ -42,6 +42,13 @@
 #include <sslpp/sslticket.h>
 #include <sslpp/sslutil.h>
 
+#include <assert.h>
+#if __cplusplus <= 199711L && !defined(static_assert)
+#define static_assert(a, b) _Static_assert(a, b)
+#endif
+
+#include <openssl/ssl.h>
+
 #include <errno.h>
 #include <limits.h>
 #include <unistd.h>
@@ -396,13 +403,13 @@ int ConfigCtx::getValidPath(char *dest, const char *path, const char *desc)
 {
     if (getAbsolutePath(dest, path) != 0)
     {
-        LS_ERROR(INVAL_PATH, desc,  path);
+        LS_WARN(INVAL_PATH, desc,  path);
         return LS_FAIL;
     }
 
     if (access(dest, F_OK) != 0)
     {
-        LS_ERROR(INACCESSIBLE_PATH, desc,  dest);
+        LS_WARN(INACCESSIBLE_PATH, desc,  dest);
         return LS_FAIL;
     }
 
