@@ -48,6 +48,8 @@ private:
     int             m_iDisableLogRotate;
     StringList      m_sSuspendedVhosts;
     int             m_iDisableWebAdmin;
+    int             m_iConfTestMode;
+    int             s_fdCmd;
     
     app_node_st  m_stExtAppXmlNodeS[MAX_EXT_APP_NUMBER];
     int             m_iEextAppXmlNodeSSize;
@@ -62,6 +64,8 @@ private:
         , m_iEnableCoreDump(0)
         , m_iDisableLogRotate(0)
         , m_iDisableWebAdmin(0)
+        , m_iConfTestMode(0)
+        , s_fdCmd(-1)
         , m_iEextAppXmlNodeSSize(0)
     {}
 public:
@@ -77,7 +81,9 @@ public:
     void setAutoIndexURI(const char *pAutoIndexURI) {   m_sAutoIndexURI = pAutoIndexURI;}
     void setEnableCoreDump(int iEnableCoreDump)     {  m_iEnableCoreDump = iEnableCoreDump; }
     void setDisableLogRotateAtStartup(int n)        {  m_iDisableLogRotate = n; }
-
+    void setConfTestMode(int n)                  {   m_iConfTestMode = n;      }
+    void setGlobalfdCmd(int n)                  { s_fdCmd = n; }
+    
     const char *getServerName()    const   {   return m_sServerName.c_str();   }
     const char *getAdminEmails()   const   {   return m_sAdminEmails.c_str();  }
     const char *getGDBPath()       const   {   return m_gdbPath.c_str();       }
@@ -94,7 +100,9 @@ public:
 
     void setDisableWebAdmin(int v)         {   m_iDisableWebAdmin = v;          }
     int getDisableWebAdmin() const         {   return m_iDisableWebAdmin;            }
-    
+    int getConfTestMode() const          { return m_iConfTestMode;               }
+    int getGlobalfdCmd() const            { return s_fdCmd; }
+
     int insertExtAppXmlNode(app_node_st *key_pointer)
     {
         if (m_iEextAppXmlNodeSSize < MAX_EXT_APP_NUMBER - 1)
